unit UMainModule;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, LResources, UHTTPServerCGI, UHTTPSessionMySQL,
  UHTTPServer, USqlDatabase, INIFiles, DateUtils, UWebPage, UWebApp,
  UXmlClasses, UHtmlClasses, UCore, UHTTPServerTCP,
  UHTTPSessionFile, UUser, SpecializedList, Registry;

const
  ConfigFile = 'Config.ini';

type
  TSession = class

  end;

  { TMainModule }

  TMainModule = class(TDataModule)
    HTTPSessionStorageMySQL1: THTTPSessionStorageMySQL;
    Database: TSqlDatabase;
    WebApp1: TWebApp;
    procedure WebApp1PageProduce(HandlerData: THTTPHandlerData);
  private
    procedure Footer;
    procedure InitDatabase;
    procedure Show(Content: string);
    procedure TopMenu;
  public
    TimeStart: TDateTime;
    Load: string;
    Unload: string;
    GlobalTitle: string;
    ShowRuntimeInfo: Boolean;
    Charset: string;
    Admin: string;
    AdminEmail: string;
    Style: string;
    Keywords: string;
    HtmlDocument: THtmlDocument;
    BaseURL: string;
    User: TWebUser;
    UserOnline: TWebOnlineUser;
    FormatHTML: Boolean;
    NetworkAddress: string;
    NetworkPort: Integer;
    MaxConnections: Integer;
    Pages: TPageList;
    procedure LoadUserInfo(HandlerData: THTTPHandlerData);
    procedure LoadFromRegistry;
    procedure SaveToRegistry;
    procedure GeneratePage(HandlerData: THTTPHandlerData; Page: TWebPage);
    procedure Run;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  MainModule: TMainModule;


implementation

uses
  UApplicationInfo, UInternetPage, UHostingPage, UHistoryPage, UDocumentsPage, UVoIPPage,
  ULinksPage, UPlansPage, UServerInfoPage, UWebCamPage, UUserControlPage,
  UNetworkPage, UAboutPage, UContactPage;

{ TMainModule }

procedure TMainModule.GeneratePage(HandlerData: THTTPHandlerData; Page: TWebPage);
var
  I: Integer;
  TitleTag: THtmlString;
begin
  with HandlerData do begin
    HtmlDocument.ContentLanguage := 'cs';
    GlobalTitle := 'ZděchovNET';
    HtmlDocument.Styles.Add(NavigationLink('/Style/' + Style + '/Style.css'));
    HtmlDocument.Scripts.Add(NavigationLink('/Style/' + Style + '/Global.js'));
    HtmlDocument.Scripts.Add(NavigationLink('/Style/' + Style + '/jquery.js'));

      TitleTag := THtmlString.Create;
      HtmlDocument.Body.SubItems.Insert(0, TitleTag);
      TopMenu;
      //Page.Page.OnProduce(HandlerData);
      HtmlDocument.Title := Page.Caption;
      TitleTag.Text := '<div class="TitlePanel"><span class="GlobalTitle">' + GlobalTitle +
        '</span> - ' + HtmlDocument.Title + '</div>';
      HtmlDocument.Title := GlobalTitle + ' - ' + HtmlDocument.Title;
      with HtmlDocument.AsXmlDocument do
      try
        Formated := FormatHTML;
        Response.Content.WriteString(AsString);
      finally
        Free;
      end;
  end;
end;

procedure TMainModule.InitDatabase;
var
  DbRows: TDbRows;
begin
  with Database do begin
    Connect;
  end;
  try
    DbRows := TDbRows.Create;
    Database.Query(DbRows, 'SET NAMES utf8');
  finally
    DbRows.Free;
  end;
end;

constructor TMainModule.Create(AOwner: TComponent);
begin
  inherited;
  Pages := TPageList.Create;
  HtmlDocument := THtmlDocument.Create;
  User := TWebUser.Create;
  User.Database := Database;
  UserOnline := TWebOnlineUser.Create;
  UserOnline.Database := Database;
  LoadFromRegistry;
  with Pages do begin
    RegisterPage(TInternetPage, InternetPage, 'internet');
    RegisterPage(THostingPage, HostingPage, 'hosting');
    RegisterPage(THistoryPage, HistoryPage, 'historie');
    RegisterPage(TDocumentsPage, DocumentsPage, 'dokumenty');
    RegisterPage(TVoIPPage, VoIPPage, 'voip');
    RegisterPage(TLinksPage, LinksPage, 'odkazy');
    RegisterPage(TContactPage, ContactPage, 'kontakt');
    RegisterPage(TServerInfoPage, ServerInfoPage, 'serverinfo');
    RegisterPage(TWebCamPage, WebCamPage, 'kamery');
    RegisterPage(TUserControlPage, UserControlPage, 'uzivatel');
    RegisterPage(TPlansPage, PlansPage, 'plany');
    RegisterPage(TNetworkPage, NetworkPage, 'sit');
    RegisterPage(TAboutPage, AboutPage, '');
    //RegisterPage(TUserControlPage, UserControlPage, '');
  end;
  with WebApp1 do begin
    if ServerType = stTCP then begin
      THTTPServerTCP(HTTPServer).Socket.Address := NetworkAddress;
      THTTPServerTCP(HTTPServer).Socket.Port := NetworkPort;
      THTTPServerTCP(HTTPServer).MaxConnection := MaxConnections;
    end;
  end;
end;

destructor TMainModule.Destroy;
begin
  //SaveToRegistry;
  HtmlDocument.Free;
  User.Free;
  UserOnline.Free;
  Pages.Free;
  inherited Destroy;
end;

procedure TMainModule.Footer;
var
  Tag: TXMLTag;
  AdminTag: TXMLTag;
  EmailTag: TXMLTag;
  ExecutionTimeTag: TXMLTag;
  UsedMemoryTag: TXMLTag;
  TextNode: TXmlString;
begin
  with TXmlTag(HtmlDocument.Body.SubItems.AddNew(TXmlTag.Create)) do begin
    Name := 'ul';
    Attributes.Values['class'] := 'Footer';
    with TXmlTag(SubElements.AddNew(TXmlTag.Create)) do begin
      Name := 'li';
      with TXmlString(SubElements.AddNew(TXmlString.Create)) do begin
        Text := Admin;
      end;
    end;
    with TXmlTag(SubElements.AddNew(TXmlTag.Create)) do begin
      Name := 'li';
      with TXmlString(SubElements.AddNew(TXmlString.Create)) do begin
        Text := AdminEmail;
      end;
    end;
    if ShowRuntimeInfo then begin
      with TXmlTag(SubElements.AddNew(TXmlTag.Create)) do begin
        Name := 'li';
        with TXmlString(SubElements.AddNew(TXmlString.Create)) do begin
          Text := 'Doba generování: ' +
            FloatToStr(Round(((Now - TimeStart) / OneMillisecond) * 100) / 100) + ' s / '; // + ini_get('max_execution_time') + ' s';
        end;
      end;
      with TXmlTag(SubElements.AddNew(TXmlTag.Create)) do begin
        Name := 'li';
        with TXmlString(SubElements.AddNew(TXmlString.Create)) do begin
          //Text := 'Použitá paměť: '  + System.PrefixMultiplier.AddPrefixMultipliers(memory_get_peak_usage(FALSE), 'B').' / '.ini_get('memory_limit').'B';
        end;
      end;
    end;
  end;
end;

procedure TMainModule.Show(Content: string);
begin
  //HtmlDocument.Title := ;
(*  BodyParam := '';
  if Load <> '' then BodyParam := BodyParam + ' onload="' + Load + '"';
  if Unload <> '' then BodyParam := BodyParam + ' onunload="' + Unload + '"';
  Document.Encoding := Charset;
  Document.Formated := FormatHTML;*)


(*
  Title := GlobalTitle;
  if Title <> '' then Title := Title + ' - ' + Title;
  Document.TitleTag.SubElements := Title;
  Title := '<span class="GlobalTitle">' + GlobalTitle + '</span>';
  if Title <> '' then Title := Title + ' - ' + $this.Title;

  Keywords := TXMLTag.Create('meta');
  Keywords.Attributes := array('name' => 'keywords', 'content' => implode(',', $this->Keywords));
  Keywords.ShringEmpty := False;
  Document.HeadTag.SubElements.Add(Keywords);

  Document.BodyTag.SubElements := array('<div class="TitlePanel">' + Title + '</div>',
  TopMenu, PageMenu, Content, Footer);
  Result := Document.GetOutput;        *)
end;

procedure TMainModule.TopMenu;
begin
  with THtmlString(HtmlDocument.Body.SubItems.InsertNew(1, THtmlString.Create)) do begin
    Text := '<div class="Navigation">';
    // Visitor
    Text := Text + '<ul class="MenuItem">' +
      '<li>' + MakeLink('Úvod', NavigationLink('/')) + '</li>' +
      '<li>' + MakeLink('Internet', NavigationLink('/internet/')) + '</li>' +
      '<li>' + MakeLink('Hosting', NavigationLink('/hosting/')) + '</li>' +
      '<li>' + MakeLink('VoIP', NavigationLink('/voip/')) + '</li>' +
      '<li>' + MakeLink('Síť', NavigationLink('/sit/')) + '</li>' +
      '<li>' + MakeLink('Odkazy', NavigationLink('/odkazy/')) + '</li>' +
      '<li>' + MakeLink('Kontakt', NavigationLink('/kontakt/')) + '</li>' +
      '<li>' + MakeLink('Kamery', NavigationLink('/kamery/')) + '</li>' +
      '<li><a href="https://mail.zdechov.net/">Pošta</a></li>' +
      '<li><a href="http://wiki.zdechov.net/">Wiki</a></li>' +
      '</ul><ul class="MenuItem2">'; //<li>&nbsp;</li>';
    if Assigned(MainModule.Pages.FindByName('uzivatel')) then begin
      if UserOnline.User = AnonymousUserId then begin
        Text := Text + '</ul>' +
          '<ul class="MenuItem2">' +
          '<li>' + MakeLink('Přihlášení', NavigationLink('/uzivatel/prihlaseni/')) + '</li>' +
          '<li>' + MakeLink('Registrace', NavigationLink('/uzivatel/registrace/')) + '</li>' +
          '<li></li>';
        // $Output .= '<li>'.$this->System->HTML->MakeLink($this->System->Translate('UserRegistration'), $this->System->MakeLink('UserList', 'Register')).'</li>';
      end else begin
        Text := Text + '</ul>' +
          '<ul class="MenuItem2">' +
          '<li>' + User.FullName + '</li>' +
          '<li>' + MakeLink('Odhlášení', NavigationLink('/uzivatel/odhlaseni/')) + '</li>' +
          '<li>' + MakeLink('Profil', NavigationLink('/uzivatel/profil/')) + '</li>';
      end;
    end else Text := Text + '</ul><ul class="MenuItem2">&nbsp;';
    Text := Text + '</ul></div>';
  end;
end;

procedure TMainModule.LoadUserInfo(HandlerData: THTTPHandlerData);
begin
  User.HandlerData := HandlerData;
  UserOnline.HandlerData := HandlerData;
  UserOnline.Update;
  User.Id := UserOnline.User;
  User.Load;
end;

procedure TMainModule.WebApp1PageProduce(HandlerData: THTTPHandlerData);
var
  FileName: string;
begin
  with HandlerData do begin
    if not Pages.ProducePage(HandlerData) then begin
      WebApp1.HTTPServer.FileResponse(HandlerData)
    end;
  end;
end;

procedure TMainModule.LoadFromRegistry;
const
  SectionGeneral = 'General';
  SectionDatabase = 'Database';
  SectionHTTPServer = 'HTTPServer';
begin
  with TIniFile.Create(ConfigFile) do
  try
    //RootKey := HKEY_CURRENT_USER;
    //OpenKey(ApplicationInfo.RegistryKey, True);
    Style := ReadString(SectionGeneral, 'Style', 'Basic');
    BaseURL := ReadString(SectionGeneral, 'BaseURL', 'http://localhost');
    Database.Hostname := ReadString(SectionDatabase, 'DatabaseHostName', 'localhost');
    Database.Database := ReadString(SectionDatabase, 'DatabaseDatabase', 'web');
    Database.UserName := ReadString(SectionDatabase, 'DatabaseUserName', 'user');
    Database.Password := ReadString(SectionDatabase, 'DatabasePassword', 'password');
    FormatHTML := ReadBool(SectionGeneral, 'FormatHTML', False);
    //MainModule.WebApp1.LogException := not ReadBool(SectionGeneral, 'ShowException', False);
    NetworkAddress := ReadString(SectionHTTPServer, 'NetworkAddress', 'localhost');
    NetworkPort := ReadInteger(SectionHTTPServer, 'NetworkPort', 80);
    MaxConnections := ReadInteger(SectionHTTPServer, 'MaxConnections', 10);
  finally
    Free;
  end;
end;

procedure TMainModule.SaveToRegistry;
const
  SectionGeneral = 'General';
  SectionDatabase = 'Database';
  SectionHTTPServer = 'HTTPServer';
begin
  with TIniFile.Create(ConfigFile) do
  try
    //RootKey := HKEY_CURRENT_USER;
    //OpenKey(ApplicationInfo.RegistryKey, True);
    WriteString(SectionGeneral, 'Style', Style);
    WriteString(SectionGeneral, 'BaseURL', BaseURL);
    WriteString(SectionDatabase, 'DatabaseHostName', Database.Hostname);
    WriteString(SectionDatabase, 'DatabaseDatabase', Database.Database);
    WriteString(SectionDatabase, 'DatabaseUserName', Database.UserName);
    WriteString(SectionDatabase, 'DatabasePassword', Database.Password);
    WriteBool(SectionGeneral, 'FormatHTML', FormatHTML);
    //WriteBool(SectionGeneral, 'ShowException', not MainModule.WebApp1.LogException);
    WriteString(SectionHTTPServer, 'NetworkAddress', NetworkAddress);
    WriteInteger(SectionHTTPServer, 'NetworkPort', NetworkPort);
    WriteInteger(SectionHTTPServer, 'MaxConnections', MaxConnections);
  finally
    Free;
  end;
end;

procedure TMainModule.Run;
begin
  InitDatabase;
  //Database.Query('SELECT * FROM ssss');

  WebApp1.Run;
end;


initialization
  {$I UMainModule.lrs}

end.

